using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;

using FlatRedBall;
using FlatRedBall.Input;
using FlatRedBall.Instructions;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Graphics.Animation;
using FlatRedBall.Graphics.Particle;
using FlatRedBall.Math.Geometry;
using FlatRedBall.Localization;



namespace Beefball.Screens
{
	public partial class GameScreen
	{
        int player1Score = 0;
        int player2Score = 0;

        void CustomInitialize()
		{
            AssignInput();
        }

        private void AssignInput()
        {
            if (InputManager.Xbox360GamePads[0].IsConnected)
            {
                PlayerBallInstance.MovementInput =
                    InputManager.Xbox360GamePads[0].LeftStick;
                PlayerBallInstance.BoostInput =
                    InputManager.Xbox360GamePads[0].GetButton(Xbox360GamePad.Button.A);
            }
            else
            {
                PlayerBallInstance.MovementInput =
                    InputManager.Keyboard.Get2DInput(Microsoft.Xna.Framework.Input.Keys.A,
                    Microsoft.Xna.Framework.Input.Keys.D,
                    Microsoft.Xna.Framework.Input.Keys.W,
                    Microsoft.Xna.Framework.Input.Keys.S);
                PlayerBallInstance.BoostInput =
                    InputManager.Keyboard.GetKey(Microsoft.Xna.Framework.Input.Keys.B);
            }

            if (InputManager.Xbox360GamePads[1].IsConnected)
            {
                PlayerBallInstance2.MovementInput =
                    InputManager.Xbox360GamePads[1].LeftStick;
                PlayerBallInstance2.BoostInput =
                    InputManager.Xbox360GamePads[1].GetButton(Xbox360GamePad.Button.A);
            }
            else
            {
                PlayerBallInstance2.MovementInput =
                    InputManager.Keyboard.Get2DInput(Microsoft.Xna.Framework.Input.Keys.Left,
                    Microsoft.Xna.Framework.Input.Keys.Right,
                    Microsoft.Xna.Framework.Input.Keys.Up,
                    Microsoft.Xna.Framework.Input.Keys.Down);
                PlayerBallInstance2.BoostInput = 
                    InputManager.Keyboard.GetKey(Microsoft.Xna.Framework.Input.Keys.RightShift);
            }
        }

        void CustomActivity(bool firstTimeCalled)
		{
            CollisionActivity();

        }

        private void CollisionActivity()
        {
            foreach (var wall in Walls)
            {
                float puckMass = 0;
                float wallMass = 1;
                float elasticity = 1;
                PuckInstance.CollideAgainstBounce(
                    wall, puckMass, wallMass, elasticity);
            }

            foreach (var player in PlayerBallList)
            {
                foreach (var wall in Walls)
                {
                    float playerMass = 0;
                    float wallMass = 1;
                    float elasticity = 1;
                    player.CollideAgainstBounce(
                        wall, playerMass, wallMass, elasticity);
                }

                foreach (var goal in Goals)
                {
                    float playerMass = 0;
                    float wallMass = 1;
                    float elasticity = 1;
                    player.CollideAgainstBounce(
                        goal, playerMass, wallMass, elasticity);
                }


                foreach (var otherPlayer in PlayerBallList)
                {
                    if (player != otherPlayer)
                    {
                        float playerMass = 1;
                        float elasticity = 1;
                        player.CollideAgainstBounce(
                            otherPlayer, playerMass, playerMass, elasticity);
                    }
                }

                // brackets used to limit the scope of variables
                {
                    float puckMass = .3f;
                    float playerMass = 1;
                    float elasticity = .8f;
                    PuckInstance.CollideAgainstBounce(
                        player, puckMass, playerMass, elasticity);
                }
            }

            if (PuckInstance.CollideAgainst(LeftGoal))
            {
                player1Score++;
                ReactToNewScore();
            }
            if (PuckInstance.CollideAgainst(RightGoal))
            {
                player2Score++;
                ReactToNewScore();
            }
        }

        private void ReactToNewScore()
        {
            PlayerBallInstance.X = 180;
            PlayerBallInstance.Y = 0;
            PlayerBallInstance.Velocity = Microsoft.Xna.Framework.Vector3.Zero;
            PlayerBallInstance.Acceleration = Microsoft.Xna.Framework.Vector3.Zero;

            PlayerBallInstance2.X = -180;
            PlayerBallInstance2.Y = 0;
            PlayerBallInstance2.Velocity = Microsoft.Xna.Framework.Vector3.Zero;
            PlayerBallInstance2.Acceleration = Microsoft.Xna.Framework.Vector3.Zero;

            PuckInstance.X = 0;
            PuckInstance.Y = 0;
            PuckInstance.Velocity = Microsoft.Xna.Framework.Vector3.Zero;

            ScoreHudInstance.Score1 = player1Score;
            ScoreHudInstance.Score2 = player2Score;
        }

        void CustomDestroy()
		{


		}

        static void CustomLoadStaticContent(string contentManagerName)
        {


        }

	}
}
